@@ -21,7 +21,6 @@
 
 import logging
 import datetime as dt
-import os.path
 
 import pygtk
 pygtk.require("2.0")
@@ -44,7 +43,7 @@ from edit_activity import CustomFactController
 from stats import StatsViewer
 from about import show_about
 from preferences import PreferencesEditor
-
+import widgets
 import idle
 
 import pango
@@ -127,8 +126,8 @@ class PanelButton(gtk.ToggleButton):
             else:
                 label = "%s %s" % (self.activity, self.duration)
         
-        label = stuff.escape_pango(label)
-        label = '<span gravity=\"south\">' + label + '</span>'
+        label = '<span gravity="south">%s</span>' % stuff.escape_pango(label)
+        self.label.set_markup("") #clear - seems to fix the warning
         self.label.set_markup(label)
 
     def get_pos(self):
@@ -230,10 +229,19 @@ class HamsterApplet(object):
         self._gui = stuff.load_ui_file("applet.ui")
         self.window = self._gui.get_object('hamster-window')
 
-        
-        # build the menu
-        self.init_dropdown()
+        self.new_name = widgets.ActivityEntry()
+        widgets.add_hint(self.new_name, _("Time and Name"))
+        self.get_widget("new_name_box").add(self.new_name)
+        self.new_name.connect("changed", self.on_activity_text_changed)
 
+        self.new_tags = widgets.TagsEntry()
+        widgets.add_hint(self.new_tags, _("Tags or Description"))
+        self.get_widget("new_tags_box").add(self.new_tags)
+
+        self.tag_box = widgets.TagBox(interactive = False)
+        self.get_widget("tag_box").add(self.tag_box)
+
+        
         # init today's tree
         self.setup_activity_tree()
 
@@ -303,7 +311,12 @@ class HamsterApplet(object):
         self.treeview.append_column(stuff.ActivityColumn(name=1,
                                                          description=5,
                                                          category=6))
+
+
+        tag_cell = widgets.TagCellRenderer()
+        self.treeview.append_column(gtk.TreeViewColumn(_("Tags"), tag_cell, data=7))
         
+
         duration_cell = gtk.CellRendererText()
         duration_cell.set_property("xalign", 1)
         timeColumn = gtk.TreeViewColumn(_("Duration"), duration_cell, text=3)
@@ -316,159 +329,6 @@ class HamsterApplet(object):
         self.treeview.append_column(self.edit_column)
 
 
-    def init_dropdown(self):
-        # set up drop down menu
-        self.activity_combo = self._gui.get_object('activity_combo')
-        self.activity_combo.child.connect('activate', self.on_activity_entered)
-        self.activity_combo.child.connect('key-press-event', self.on_activity_list_key_pressed)
-        self.activity_combo.child.connect('changed', self.on_activity_entry_changed)
-
-        self.activity_combo.set_model(gtk.ListStore(gobject.TYPE_STRING,
-                                                   gobject.TYPE_STRING,
-                                                   gobject.TYPE_STRING))
-        self.activity_combo.set_text_column(2)
-        self.activity_combo.clear()
-
-
-        activity_cell = gtk.CellRendererText()
-        self.activity_combo.pack_start(activity_cell, True)
-        self.activity_combo.add_attribute(activity_cell, 'text', 0)
-
-        category_cell = stuff.CategoryCell()  
-        self.activity_combo.pack_start(category_cell, False)
-        self.activity_combo.add_attribute(category_cell, 'text', 1)
-        
-
-        #now set up completion
-        self.completion = gtk.EntryCompletion()
-        #fillable, activity_name, category
-        self.completion.set_model(gtk.ListStore(gobject.TYPE_STRING,
-                                                gobject.TYPE_STRING,
-                                                gobject.TYPE_STRING))
-        self.completion.set_minimum_key_length(1)
-        self.completion.set_inline_completion(True)
-        self.completion.set_popup_set_width(False)
-        self.completion.set_popup_single_match(False)
-        self.completion.set_text_column(0)
-        self.activity_combo.child.set_completion(self.completion)
-        
-        self.refresh_dropdown()
-
-
-    def refresh_dropdown(self):        
-        self.all_activities = runtime.storage.get_autocomplete_activities()
-        self.all_categories = runtime.storage.get_category_list()
-        self.eds_tasks = eds.get_eds_tasks()
-
-        #add evolution tasks to dropdown, yay!
-        for activity in self.eds_tasks:
-            self.all_activities.append(activity)
-
-        #now populate the menu - contains only categorized entries
-        store = self.activity_combo.get_model()
-        store.clear()
-
-        categorized_activities = runtime.storage.get_sorted_activities()
-        for activity in categorized_activities:
-            activity_category = activity['name']
-            if activity['category']:
-                activity_category += "@%s" % activity['category']
-
-            store.append([activity['name'],
-                          activity['category'],
-                          activity_category])
-
-
-    def on_match_selected(self, completion, model, iter):
-        entry = completion.get_entry()
-        activity = stuff.parse_activity_input(entry.get_text())
-        
-        fillable = model.get_value(iter, 0)
-        
-        res = ""
-        
-        if activity.start_time:
-            res += "%s" % activity.start_time.strftime("%H:%M")
-            if activity.end_time:
-                res += "-%s" % activity.start_time.strftime("%H:%M")
-        
-            res += " "
-            #strip time
-            fillable = fillable[fillable.find(" ")+1:]
-        
-        res += fillable
-        
-        entry.set_text(res)
-        entry.set_position(len(res))
-        return True
-    
-    def redo_autocomplete(self):
-        """parses input and generates autocomplete according to what has
-        to be completed now"""
-        #TODO turn this whole thing into a widget
-        store = self.completion.get_model()
-        store.clear()
-        
-        input_text = self.activity_combo.child.get_text()
-        
-        if not input_text:
-            return
-        
-        
-        entry = self.activity_combo.child
-        parsed_activity = stuff.parse_activity_input(entry.get_text())
-
-        if input_text.find("@") > 0:
-            key = input_text[input_text.find("@")+1:].lower()
-            for category in self.all_categories:
-                if key in category['name'].lower():
-                    fillable = (input_text[:input_text.find("@") + 1] + category['name']).lower()
-                    store.append([fillable, category['name'], fillable])
-
-        else:
-            for activity in self.all_activities:
-                fillable = activity['name']
-                if activity['category']:
-                    fillable += "@%s" % activity['category']
-                    
-                if parsed_activity.start_time:
-                    fillable = entry.get_text()[:entry.get_text().find(" ")+1] + fillable
-    
-                store.append([fillable, activity['name'], activity['category']])
-
-                
-        activity = stuff.parse_activity_input(input_text)
-        self.completion.clear()
-        if activity.start_time:
-            bgcolor = gtk.Style().bg[gtk.STATE_NORMAL].to_string()
-            time_cell = gtk.CellRendererPixbuf()
-            time_cell.set_property("icon-name", "appointment-new")
-            time_cell.set_property("cell-background", bgcolor)
-            
-            self.completion.pack_start(time_cell, False)
-
-            time_cell = gtk.CellRendererText()
-            time = activity.start_time.strftime("%H:%M")
-            if activity.end_time:
-                time += "-%s" % activity.end_time.strftime("%H:%M")
-                
-            time_cell.set_property("text", time)
-            time_cell.set_property("scale", 0.8)
-            time_cell.set_property("cell-background", bgcolor)
-            self.completion.pack_start(time_cell, False)
-
-        activity_cell = gtk.CellRendererText()
-        self.completion.pack_start(activity_cell, True)
-        self.completion.add_attribute(activity_cell, 'text', 1)
-        #self.completion.set_property("text-column", 2)
-
-        if input_text.find("@") == -1:
-            #no need for category cell when we populate categories
-            category_cell = stuff.CategoryCell()  
-            self.completion.pack_start(category_cell, False)
-            self.completion.add_attribute(category_cell, 'text', 2)
-        
-
     """UI functions"""
     def refresh_hamster(self):
         """refresh hamster every x secs - load today, check last activity etc."""        
@@ -489,14 +349,45 @@ class HamsterApplet(object):
                                stuff.format_duration(duration, False))
             self.button.set_text(self.last_activity['name'],
                                  stuff.format_duration(duration, False))
-            
-            self._gui.get_object('stop_tracking').set_sensitive(1);
         else:
             label = "%s" % _(u"No activity")
             self.button.set_text(label, None)
-            self._gui.get_object('stop_tracking').set_sensitive(0);
+            
+        self.set_last_activity()
+
+        
+    def set_last_activity(self):
+        activity = runtime.storage.get_last_activity()
+        self.get_widget("stop_tracking").set_sensitive(activity != None)
+        self.get_widget("edit_current_activity").set_sensitive(activity != None)
         
         
+        if activity:
+            self.get_widget("switch_activity").show()
+            self.get_widget("start_tracking").hide()
+            
+            delta = dt.datetime.now() - activity['start_time']
+            duration = delta.seconds /  60
+            
+            self.get_widget("last_activity_duration").set_text(stuff.format_duration(duration) or _("Just started"))
+            self.get_widget("last_activity_name").set_text(activity['name'])
+            if activity['category'] != _("Unsorted"):
+                self.get_widget("last_activity_category") \
+                    .set_text(" - %s" % activity['category'])
+
+            self.get_widget("last_activity_description").set_text(activity['description'] or "")
+
+            self.tag_box.draw(activity["tags"])
+        else:
+            self.get_widget("switch_activity").hide()
+            self.get_widget("start_tracking").show()
+
+            self.get_widget("last_activity_name").set_text(_("No activity"))
+            self.get_widget("last_activity_duration").set_text("")
+            self.get_widget("last_activity_category").set_text("")
+            self.tag_box.draw([])
+            self.get_widget("last_activity_description").set_text("")
+         
     def check_user(self):
         if not self.notify_interval: #no interval means "never"
             return
@@ -536,8 +427,14 @@ class HamsterApplet(object):
         fact_store.clear()
         facts = runtime.storage.get_facts(today)
         
+        if len(facts) > 10:
+            self._gui.get_object("todays_scroll").set_size_request(-1, 250)
+            self._gui.get_object("todays_scroll").set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+        else:
+            self._gui.get_object("todays_scroll").set_size_request(-1, -1)
+            self._gui.get_object("todays_scroll").set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+            
         by_category = {}
-        
         for fact in facts:
             duration = 24 * 60 * fact["delta"].days + fact["delta"].seconds / 60
             by_category[fact['category']] = \
@@ -606,19 +503,6 @@ class HamsterApplet(object):
         """main window display and positioning"""
         self.button.set_active(is_active)
 
-        if self.last_activity and self.last_activity["end_time"] is None:
-            label = self.last_activity['name']
-            if self.last_activity['category'] != _("Unsorted"):
-                label += "@%s" %  self.last_activity['category']
-            self.activity_combo.child.set_text(label)
-
-            self.activity_combo.child.select_region(0, -1)
-            self._gui.get_object("more_info_label").hide()
-        else:
-            self.activity_combo.child.set_text('')
-            self._gui.get_object("more_info_label").show()
-
-
         if not is_active:
             self.window.hide()
             return
@@ -650,13 +534,15 @@ class HamsterApplet(object):
         self.window.unstick()
         self.window.stick() #show on all desktops
 
+        self.new_name.set_text("");
+        self.new_tags.set_text("");
         gobject.idle_add(self._delayed_display)  
         
     def _delayed_display(self):
         """show window only when gtk has become idle. otherwise we get
         mixed results. TODO - this looks like a hack though"""
         self.window.present()
-        self.activity_combo.grab_focus()
+        self.new_name.grab_focus()
         
 
     """events"""
@@ -675,32 +561,9 @@ class HamsterApplet(object):
     def on_toggle(self, widget):
         self.__show_toggle(None, self.button.get_active())
 
-    def on_activity_list_key_pressed(self, entry, event):
-        #tab will trigger going through autocomplete values when there are any
-        if event.keyval == gtk.keysyms.Tab:
-            event.keyval = gtk.keysyms.Down
-        
-        #down will trigger showing dropdown instead of selecting first whatever
-        elif event.keyval == gtk.keysyms.Down and len(self.completion.get_model()) == 0:
-            self.activity_combo.popup()
-            return True
-        
-        return False
-    
-    def on_activity_entry_changed(self, entry):
-        self.redo_autocomplete()
-        return False
-        
-    def on_activity_switched(self, component):
-        # do stuff only if user has selected something
-        # for other cases activity_edited will be triggered
-        if component.get_active_iter():
-            component.child.activate() # forward
-        return True
-
     def on_activity_entered(self, component):
         """fires, when user writes activity by hand"""
-        activity_name = component.get_text().decode('utf8', 'replace')
+        activity_name = self.new_name.get_text().decode('utf8', 'replace')
         
         if activity_name == "":
             return
@@ -727,7 +590,12 @@ class HamsterApplet(object):
         fact_id = model[iter][0]
             
         custom_fact = CustomFactController(self, None, fact_id)
-        custom_fact.show()        
+        custom_fact.show()
+        
+    def on_edit_current_activity_clicked(self, widget):
+        if self.last_activity:
+            custom_fact = CustomFactController(self, None, self.last_activity["id"])
+            custom_fact.show()
     
     def on_today_row_activated(self, tree, path, column):
         if column == self.edit_column:
@@ -742,10 +610,11 @@ class HamsterApplet(object):
                 if fact['category']:
                     activity = '%s@%s' % (activity, fact['category'])
 
-                if fact['description']:
-                    activity = "%s, %s" % (activity, fact['description'])
+                tags = fact["tags"]
+                if fact["description"]:
+                    tags.append(fact["description"])
                     
-                runtime.storage.add_fact(activity)
+                runtime.storage.add_fact(activity, ", ".join(tags))
                 runtime.dispatcher.dispatch('panel_visible', False)
         
         
@@ -753,17 +622,13 @@ class HamsterApplet(object):
         if (event_key.keyval == gtk.keysyms.Escape
           or (event_key.keyval == gtk.keysyms.w 
               and event_key.state & gtk.gdk.CONTROL_MASK)):
-            runtime.dispatcher.dispatch('panel_visible', False)
-            return True
+            if self.new_name.popup.get_property("visible") == False \
+               and self.new_tags.popup.get_property("visible") == False:
+                runtime.dispatcher.dispatch('panel_visible', False)
+                return True
         return False
         
     """button events"""
-    def on_stop_tracking(self, button):
-        runtime.storage.touch_fact(self.last_activity)
-        self.last_activity = None
-        self.update_label()
-        runtime.dispatcher.dispatch('panel_visible', False)
-
     def on_overview(self, menu_item):
         runtime.dispatcher.dispatch('panel_visible', False)
         stats_viewer = StatsViewer(self)
@@ -793,7 +658,7 @@ class HamsterApplet(object):
     
     """signals"""
     def after_activity_update(self, widget, renames):
-        self.refresh_dropdown()
+        self.new_name.refresh_activities()
         self.load_day()
         self.update_label()
     
@@ -860,3 +725,22 @@ Now, start tracking!
         message_dialog.set_markup(more_info)
         message_dialog.show()
 
+    def on_activity_text_changed(self, widget):
+        self.get_widget("switch_activity").set_sensitive(widget.get_text() != "")
+
+    def on_switch_activity_clicked(self, widget):
+        runtime.storage.add_fact(self.new_name.get_text().encode("utf-8"), self.new_tags.get_text())
+        self.new_name.set_text("")
+        self.new_tags.set_text("")
+        runtime.dispatcher.dispatch('panel_visible', False)
+
+    def on_stop_tracking_clicked(self, widget):
+        runtime.storage.touch_fact(runtime.storage.get_last_activity())
+        self.last_activity = None
+        runtime.dispatcher.dispatch('panel_visible', False)
+
+    def show(self):
+        self.window.show_all()
+        
+    def get_widget(self, name):
+        return self._gui.get_object(name)
