@@ -26,11 +26,13 @@ import os.path
 import gettext, locale
 import gnome
 import logging
+import gobject
 
 def applet_factory(applet, iid):
     applet.connect("destroy", on_destroy)
     applet.set_applet_flags(gnomeapplet.EXPAND_MINOR)
 
+    from hamster.applet import HamsterApplet    
     hamster_applet = HamsterApplet(applet)
 
     applet.show_all()
@@ -108,10 +110,10 @@ if __name__ == "__main__":
             if hasattr(module, 'bind_textdomain_codeset'):
                 module.bind_textdomain_codeset('hamster-applet','UTF-8')
         
-        from hamster.applet import HamsterApplet    
         gtk.window_set_default_icon_name("hamster-applet")
     
         if options.start_window or options.standalone:
+            gobject.set_application_name("hamster-applet")
             if options.start_window == "stats":
                 from hamster.stats import StatsViewer
                 stats_viewer = StatsViewer().show()
